@@ -71,7 +71,7 @@ from . import xla
 from tensorflow.python.data.ops.optional_ops import OptionalSpec
 from tensorflow.python.eager.backprop import GradientTape
 from tensorflow.python.eager.context import executing_eagerly
-from tensorflow.python.eager.def_function import function
+from tensorflow.python.eager.polymorphic_function.polymorphic_function import function
 from tensorflow.python.framework.constant_op import constant
 from tensorflow.python.framework.device_spec import DeviceSpecV2 as DeviceSpec
 from tensorflow.python.framework.dtypes import DType
@@ -111,18 +111,19 @@ from tensorflow.python.framework.ops import Operation
 from tensorflow.python.framework.ops import RegisterGradient
 from tensorflow.python.framework.ops import Tensor
 from tensorflow.python.framework.ops import control_dependencies
-from tensorflow.python.framework.ops import convert_to_tensor_v2_with_dispatch as convert_to_tensor
 from tensorflow.python.framework.ops import device_v2 as device
 from tensorflow.python.framework.ops import get_current_name_scope
 from tensorflow.python.framework.ops import init_scope
 from tensorflow.python.framework.ops import inside_function
+from tensorflow.python.framework.ops import is_symbolic_tensor
 from tensorflow.python.framework.ops import name_scope_v2 as name_scope
 from tensorflow.python.framework.ops import no_gradient
 from tensorflow.python.framework.sparse_tensor import SparseTensor
 from tensorflow.python.framework.sparse_tensor import SparseTensorSpec
+from tensorflow.python.framework.tensor import TensorSpec
+from tensorflow.python.framework.tensor_conversion import convert_to_tensor_v2_with_dispatch as convert_to_tensor
 from tensorflow.python.framework.tensor_conversion_registry import register_tensor_conversion_function
 from tensorflow.python.framework.tensor_shape import TensorShape
-from tensorflow.python.framework.tensor_spec import TensorSpec
 from tensorflow.python.framework.tensor_util import MakeNdarray as make_ndarray
 from tensorflow.python.framework.tensor_util import constant_value as get_static_value
 from tensorflow.python.framework.tensor_util import is_tf_type as is_tensor
@@ -131,6 +132,7 @@ from tensorflow.python.framework.type_spec import TypeSpec
 from tensorflow.python.framework.type_spec import type_spec_from_value
 from tensorflow.python.framework.versions import COMPILER_VERSION as __compiler_version__
 from tensorflow.python.framework.versions import CXX11_ABI_FLAG as __cxx11_abi_flag__
+from tensorflow.python.framework.versions import CXX_VERSION as __cxx_version__
 from tensorflow.python.framework.versions import GIT_VERSION as __git_version__
 from tensorflow.python.framework.versions import MONOLITHIC_BUILD as __monolithic_build__
 from tensorflow.python.framework.versions import VERSION as __version__
@@ -169,17 +171,17 @@ from tensorflow.python.ops.array_ops import slice
 from tensorflow.python.ops.array_ops import space_to_batch_v2 as space_to_batch
 from tensorflow.python.ops.array_ops import split
 from tensorflow.python.ops.array_ops import squeeze_v2 as squeeze
-from tensorflow.python.ops.array_ops import stack
 from tensorflow.python.ops.array_ops import stop_gradient
 from tensorflow.python.ops.array_ops import strided_slice
 from tensorflow.python.ops.array_ops import tensor_scatter_nd_update
 from tensorflow.python.ops.array_ops import transpose_v2 as transpose
 from tensorflow.python.ops.array_ops import unique
 from tensorflow.python.ops.array_ops import unique_with_counts
-from tensorflow.python.ops.array_ops import unstack
 from tensorflow.python.ops.array_ops import where_v2 as where
 from tensorflow.python.ops.array_ops import zeros
 from tensorflow.python.ops.array_ops import zeros_like_v2 as zeros_like
+from tensorflow.python.ops.array_ops_stack import stack
+from tensorflow.python.ops.array_ops_stack import unstack
 from tensorflow.python.ops.batch_ops import batch_function as nondifferentiable_batch_function
 from tensorflow.python.ops.check_ops import assert_equal_v2 as assert_equal
 from tensorflow.python.ops.check_ops import assert_greater_v2 as assert_greater
@@ -189,13 +191,12 @@ from tensorflow.python.ops.check_ops import ensure_shape
 from tensorflow.python.ops.clip_ops import clip_by_global_norm
 from tensorflow.python.ops.clip_ops import clip_by_norm
 from tensorflow.python.ops.clip_ops import clip_by_value
-from tensorflow.python.ops.control_flow_ops import Assert
-from tensorflow.python.ops.control_flow_ops import case_v2 as case
-from tensorflow.python.ops.control_flow_ops import cond_for_tf_v2 as cond
+from tensorflow.python.ops.cond import cond_for_tf_v2 as cond
+from tensorflow.python.ops.control_flow_assert import Assert
+from tensorflow.python.ops.control_flow_case import case_v2 as case
 from tensorflow.python.ops.control_flow_ops import group
-from tensorflow.python.ops.control_flow_ops import switch_case
 from tensorflow.python.ops.control_flow_ops import tuple_v2 as tuple
-from tensorflow.python.ops.control_flow_ops import while_loop_v2 as while_loop
+from tensorflow.python.ops.control_flow_switch_case import switch_case
 from tensorflow.python.ops.critical_section_ops import CriticalSection
 from tensorflow.python.ops.custom_gradient import custom_gradient
 from tensorflow.python.ops.custom_gradient import grad_pass_through
@@ -248,6 +249,10 @@ from tensorflow.python.ops.gen_math_ops import tanh
 from tensorflow.python.ops.gen_math_ops import truncate_div as truncatediv
 from tensorflow.python.ops.gen_math_ops import truncate_mod as truncatemod
 from tensorflow.python.ops.gen_nn_ops import approx_top_k
+from tensorflow.python.ops.gen_nn_ops import conv2d_backprop_filter_v2
+from tensorflow.python.ops.gen_nn_ops import conv2d_backprop_input_v2
+from tensorflow.python.ops.gen_ragged_array_ops import ragged_fill_empty_rows
+from tensorflow.python.ops.gen_ragged_array_ops import ragged_fill_empty_rows_grad
 from tensorflow.python.ops.gen_random_index_shuffle_ops import random_index_shuffle
 from tensorflow.python.ops.gen_string_ops import as_string
 from tensorflow.python.ops.gradients_impl import HessiansV2 as hessians
@@ -318,6 +323,7 @@ from tensorflow.python.ops.variable_scope import variable_creator_scope
 from tensorflow.python.ops.variables import Variable
 from tensorflow.python.ops.variables import VariableAggregationV2 as VariableAggregation
 from tensorflow.python.ops.variables import VariableSynchronization
+from tensorflow.python.ops.while_loop import while_loop_v2 as while_loop
 from tensorflow.python.platform.tf_logging import get_logger
 
 # WRAPPER_PLACEHOLDER
