@@ -14,10 +14,6 @@
 # ==============================================================================
 """SavedModel builder implementation."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import functools
 import os
 
@@ -32,9 +28,10 @@ from tensorflow.python.framework import ops
 from tensorflow.python.lib.io import file_io
 from tensorflow.python.ops import variables
 from tensorflow.python.platform import tf_logging
-from tensorflow.python.saved_model import constants
+from tensorflow.python.saved_model import fingerprinting_utils
+from tensorflow.python.saved_model import path_helpers
 from tensorflow.python.saved_model import signature_def_utils
-from tensorflow.python.saved_model import utils_impl as saved_model_utils
+from tensorflow.python.saved_model.pywrap_saved_model import constants
 from tensorflow.python.saved_model.pywrap_saved_model import metrics
 from tensorflow.python.training import saver as tf_saver
 from tensorflow.python.util import compat
@@ -115,6 +112,7 @@ class _SavedModelBuilder(object):
     # on the SavedModel MUST use the add_meta_graph() API which does not save
     # weights.
     self._has_saved_variables = False
+    self._saved_asset_files = set()
 
   def _save_and_write_assets(self, meta_graph_def, assets_list=None):
     """Saves asset to the meta graph and writes asset files to disk.
@@ -133,7 +131,8 @@ class _SavedModelBuilder(object):
       return
 
     # Copy assets from source path to destination path.
-    copy_assets_to_destination_dir(asset_filename_map, self._export_dir)
+    copy_assets_to_destination_dir(asset_filename_map, self._export_dir,
+                                   self._saved_asset_files)
 
   def _tag_and_add_meta_graph(self, meta_graph_def, tags, signature_def_map):
     """Tags the meta graph def and adds it to the SavedModel.
@@ -145,7 +144,7 @@ class _SavedModelBuilder(object):
       meta_graph_def: The meta graph def to add to the SavedModel.
       tags: The set of tags to annotate the meta graph def with.
       signature_def_map: The map of signature defs to be added to the meta graph
-          def.
+        def.
     """
     for tag in tags:
       meta_graph_def.meta_info_def.tags.append(tag)
@@ -249,15 +248,15 @@ class _SavedModelBuilder(object):
     Args:
       tags: The set of tags to annotate the meta graph def with.
       signature_def_map: The map of signature defs to be added to the meta graph
-          def.
+        def.
       assets_list: Assets to be saved with SavedModel. Note
           that this list should be a subset of the assets saved as part of
           the first meta graph in the SavedModel.
       clear_devices: Set to true if the device info on the default graph should
-          be cleared.
+        be cleared.
       init_op: Op or group of ops to execute when the graph is loaded. Note
           that when the init_op is specified it is run after the restore op at
-          load-time.
+        load-time.
       train_op: Op or group of opts that trains the model when run. This will
         not be run automatically when the graph is loaded, instead saved in
         a SignatureDef accessible through the exported MetaGraph.
@@ -331,10 +330,10 @@ class _SavedModelBuilder(object):
         def.
       assets_list: Assets to be saved with SavedModel.
       clear_devices: Set to true if the device info on the default graph should
-          be cleared.
+        be cleared.
       init_op: Op or group of ops to execute when the graph is loaded. Note
           that when the init_op is specified it is run after the restore op at
-          load-time.
+        load-time.
       train_op: Op or group of ops that trains the model when run. This will
         not be run automatically when the graph is loaded, instead saved in
         a SignatureDef accessible through the exported MetaGraph.
@@ -364,8 +363,8 @@ class _SavedModelBuilder(object):
     _add_op_to_signature_def_map(signature_def_map, train_op,
                                  constants.TRAIN_OP_SIGNATURE_KEY)
 
-    saved_model_utils.get_or_create_variables_dir(self._export_dir)
-    variables_path = saved_model_utils.get_variables_path(self._export_dir)
+    path_helpers.get_or_create_variables_dir(self._export_dir)
+    variables_path = path_helpers.get_variables_path(self._export_dir)
 
     saver = self._maybe_create_saver(saver)
 
@@ -418,6 +417,9 @@ class _SavedModelBuilder(object):
     if not file_io.file_exists(self._export_dir):
       file_io.recursive_create_dir(self._export_dir)
 
+    saved_model_serialized = self._saved_model.SerializeToString(
+        deterministic=True)
+
     if as_text:
       path = file_io.join(
           compat.as_bytes(self._export_dir),
@@ -428,9 +430,12 @@ class _SavedModelBuilder(object):
           compat.as_bytes(self._export_dir),
           compat.as_bytes(constants.SAVED_MODEL_FILENAME_PB))
       file_io.write_string_to_file(
-          path, self._saved_model.SerializeToString(deterministic=True))
+          path, saved_model_serialized)
     tf_logging.info("SavedModel written to: %s", compat.as_text(path))
     metrics.IncrementWrite(write_version="1")
+
+    # Placeholder for internal TF1 model fingerprint write
+
     return path
 
 
@@ -468,7 +473,8 @@ class SavedModelBuilder(_SavedModelBuilder):
       return
 
     # Copy assets from source path to destination path.
-    copy_assets_to_destination_dir(asset_filename_map, self._export_dir)
+    copy_assets_to_destination_dir(asset_filename_map, self._export_dir,
+                                   self._saved_asset_files)
 
   def _maybe_add_main_op(self, main_op):
     """Adds main op to the SavedModel.
@@ -591,8 +597,8 @@ class SavedModelBuilder(_SavedModelBuilder):
     # Add assets and ops
     self._add_collections(assets_collection, main_op, None)
 
-    saved_model_utils.get_or_create_variables_dir(self._export_dir)
-    variables_path = saved_model_utils.get_variables_path(self._export_dir)
+    path_helpers.get_or_create_variables_dir(self._export_dir)
+    variables_path = path_helpers.get_variables_path(self._export_dir)
 
     saver = self._maybe_create_saver(saver)
 
@@ -763,9 +769,21 @@ def _add_asset_to_metagraph(meta_graph_def, asset_filename, asset_tensor):
   asset_proto.tensor_info.name = asset_tensor.name
 
 
-def copy_assets_to_destination_dir(asset_filename_map, destination_dir):
-  """Copy all assets from source path to destination path."""
-  assets_destination_dir = saved_model_utils.get_or_create_assets_dir(
+def copy_assets_to_destination_dir(asset_filename_map, destination_dir,
+                                   saved_files=None):
+  """Copy all assets from source path to destination path.
+
+  Args:
+    asset_filename_map: a dict of filenames used for saving the asset in
+      the SavedModel to full paths from which the filenames were derived.
+    destination_dir: the destination directory that assets are stored in.
+    saved_files: a set of destination filepaths that have already been copied
+      and will be skipped
+  """
+  if saved_files is None:
+    saved_files = set()
+
+  assets_destination_dir = path_helpers.get_or_create_assets_dir(
       destination_dir)
 
   # Copy each asset from source path to destination path.
@@ -774,11 +792,14 @@ def copy_assets_to_destination_dir(asset_filename_map, destination_dir):
         compat.as_bytes(assets_destination_dir),
         compat.as_bytes(asset_basename))
 
-    # Only copy the asset file to the destination if it does not already
-    # exist. This is to ensure that an asset with the same name defined as
-    # part of multiple graphs is only copied the first time.
-    if not file_io.file_exists(asset_destination_filepath):
-      file_io.copy(asset_source_filepath, asset_destination_filepath)
+    # Copy if source file exists, src & dst are not the same, and dst is not in
+    # saved_files
+    if (file_io.file_exists(asset_source_filepath) and
+        asset_source_filepath != asset_destination_filepath and
+        asset_destination_filepath not in saved_files):
+      file_io.copy(
+          asset_source_filepath, asset_destination_filepath, overwrite=True)
+      saved_files.add(asset_destination_filepath)
 
   tf_logging.info("Assets written to: %s",
                   compat.as_text(assets_destination_dir))
